@@ -1,15 +1,25 @@
-import shutil
 import uuid
 import re
 from copy import deepcopy
 from typing import Any, Optional, Union
 
-from nbmanips.cell_utils import printable_cell
+try:
+    import pygments
+    from pygments.lexers import get_lexer_by_name
+
+    _MARKDOWN_LEXER = get_lexer_by_name('md')
+except ImportError:
+    pygments = None
+    get_lexer_by_name = None
+
+from nbmanips.cell_utils import printable_cell, FORMATTER
 from nbmanips.cell_output import CellOutput
 from nbmanips.utils import total_size
 
 
 class Cell:
+    _cell_types = {}
+
     def __init__(self, content, num=None):
         self.cell = content
         self._num = num
@@ -46,7 +56,11 @@ class Cell:
 
     @property
     def output(self):
-        return self.get_output(text=True, readable=True).strip()
+        return self.get_output(text=True).strip()
+
+    @property
+    def outputs(self):
+        return map(CellOutput, self.cell.get('outputs', []))
 
     def get_copy(self, new_id=None):
         cell = self.__class__(deepcopy(self.cell), None)
@@ -54,20 +68,21 @@ class Cell:
             cell.id = new_id
         return cell
 
-    def get_output(self, text=True, readable=True, exclude_data_types=None, exclude_errors=True, **kwargs):
+    def get_output(self, text=True, parsers=None, parsers_config=None, excluded_data_types=None):
         """
         Tries its best to return a readable output from cell
 
-        :param exclude_data_types:
-        :param exclude_errors:
         :param text:
-        :param readable:
+        :param parsers: parsers to use
+        :param parsers_config: custom config for parsers
+        :param excluded_data_types: data types to exclude
         :return:
         """
+
         outputs = []
-        for output in self.cell.get('outputs', []):
-            cell_output = CellOutput.new(output)
-            outputs.append(cell_output.to_str(readable, exclude_data_types))
+        for cell_output in self.outputs:
+            text_output = cell_output.to_str(parsers, parsers_config, excluded_data_types)
+            outputs.append(text_output)
 
         if text:
             return '\n'.join(outputs)
@@ -106,7 +121,8 @@ class Cell:
         :param output_types: Output Type(MIME type) to delete: text/plain, text/html, image/png, ...
         :type output_types: set or str or None to delete all output
         """
-        if self.type != "code":
+        outputs = list(self.outputs)
+        if len(outputs) == 0:
             return
 
         if output_types is None:
@@ -118,8 +134,7 @@ class Cell:
             output_types = set(output_types)
 
         new_outputs = []
-        for output in self['outputs']:
-            cell_output = CellOutput.new(output)
+        for cell_output in outputs:
             new_output = cell_output.erase_output(output_types)
             if new_output is not None:
                 new_outputs.append(new_output)
@@ -134,10 +149,7 @@ class Cell:
         :type output_types: set
         :return: a bool object (True if cell should be selected)
         """
-        if self.type != "code":
-            return False
-
-        return any(CellOutput.new(output).has_output_type(output_types) for output in self['outputs'])
+        return any(cell_output.has_output_type(output_types) for cell_output in self.outputs)
 
     def byte_size(self, output_types: Optional[set] = None, ignore_source=False):
         """
@@ -149,22 +161,11 @@ class Cell:
         :return: a bool object (True if cell should be selected)
         """
         size = 0 if ignore_source else total_size(self['source'])
-        size += sum([CellOutput.new(output).byte_size(output_types) for output in self['outputs']])
+        size += sum([cell_output.byte_size(output_types) for cell_output in self.outputs])
         return size
 
-    def to_str(self, width=None, style='single', color=None, img_color=None, img_width=None):
-        if self.type == 'code':
-            width = width or (shutil.get_terminal_size().columns - 1)
-            img_width = img_width if img_width else int(width*0.8)
-            sources = [printable_cell(self.source, width=width, style=style, color=color)]
-
-            img_color = bool(color) if img_color is None else img_color
-            output = self.get_output(text=True, readable=True, colorful=img_color, width=img_width).strip()
-            if output:
-                sources.append(output)
-            return '\n'.join(sources)
-        else:
-            return self.source
+    def to_str(self, width=None, *args, **kwargs):
+        return self.source
 
     def show(self):
         print(self)
@@ -173,7 +174,7 @@ class Cell:
         return f"<Cell {self.num}>" if self.num else "<Cell>"
 
     def __str__(self):
-        return self.to_str(width=None, style='single', color=None, img_color=None)
+        return self.to_str(width=None, style='single', color=None)
 
     # metadata
     def update_metadata(self, key: str, value: Any):
@@ -222,3 +223,48 @@ class Cell:
     def generate_id_candidate():
         return uuid.uuid4().hex[:8]
 
+    def __new__(cls, content, *args, **kwargs):
+        cell_type = content['cell_type']
+        cell_class = cls._cell_types[cell_type]
+        obj = super().__new__(cell_class)
+        cell_class.__init__(obj, content, *args, **kwargs)
+        return obj
+
+    def __init_subclass__(cls, cell_type=None, **kwargs):
+        super().__init_subclass__(**kwargs)
+        if cell_type:
+            cls._cell_types[cell_type] = cls
+
+
+class CodeCell(Cell, cell_type="code"):
+    def to_str(self, width=None, style='single', use_pygments=None, pygments_lexer=None, color=None,
+               exclude_output=False, parsers=None, parsers_config=None, excluded_data_types=None):
+        sources = [printable_cell(self.source, width=width, style=style, color=color, pygments_lexer=pygments_lexer)]
+
+        if not exclude_output:
+            output = self.get_output(
+                text=True,
+                parsers=parsers,
+                parsers_config=parsers_config,
+                excluded_data_types=excluded_data_types,
+            ).strip()
+            if output:
+                sources.append(output)
+
+        return '\n'.join(sources)
+
+
+class MarkdownCell(Cell, cell_type="markdown"):
+    def to_str(self, width=None, style='single', use_pygments=None, pygments_lexer=None, color=None, **kwargs):
+        use_pygments = pygments is not None if use_pygments is None else use_pygments
+
+        if use_pygments:
+            if pygments is None:
+                raise ModuleNotFoundError("You need to install pygments first.\n pip install pygments")
+            return pygments.highlight(self.source, _MARKDOWN_LEXER, FORMATTER)[:-1]
+        else:
+            return self.source
+
+
+class RawCell(Cell, cell_type="raw"):
+    pass
